// -*- C++ -*-
//
//  This file is part of CBCBSIM, the Combined Broadcast Content-Based
//  (CBCB) routing simulation.
//
//  Authors: See the file AUTHORS for full details. 
//
//  Copyright (C) 2002-2004 University of Colorado
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,
//  USA, or send email to serl@cs.colorado.edu.
//
// $Id: CBCBNode.h,v 1.4 2008-04-01 17:02:30 carzanig Exp $
//
#ifndef CBCBSIM_CBCBNODE_H
#define CBCBSIM_CBCBNODE_H

#include <cbnsim/Node.h>
#include <cbnsim/types.h>
#include <siena/forwarding.h>
#include "cbcbsim/BroadcastLayer.h"
#include "cbcbsim/ME.h"
#include "cbcbsim/ProtocolOptions.h"
#include "cbcbsim/RA.h"
#include "cbcbsim/SR.h"
#include "cbcbsim/UR.h"

namespace cbcbsim
{
  /** @brief Node class for CBCB protocol.
   */
  class CBCBNode : public cbnsim::Node
  {
    class sr_response
    {
    public:
      std::set<ssim::ProcessId> respondees;
      cbnsim::Predicate pred;
    };

    void process_me( unsigned int ifid, const ME& me );
    void process_ra( unsigned int ifid, const RA& ra );
    void process_sr( unsigned int ifid, const SR& sr );
    void process_ur( unsigned int ifid, const UR& ur );
    bool propagate_me( ssim::ProcessId issuer, cbnsim::MessageId msg_id, const Message& msg );
    void propagate_ra( ssim::ProcessId issuer, const cbnsim::Predicate& pred );
    unsigned int propagate_sr( ssim::ProcessId issuer, unsigned int number );
    void update_fwd_table();

    const BroadcastLayer&		m_bl;
    siena::FwdTable			m_fwd_table;
    bool				m_issue_pending;
    std::vector<unsigned int>		m_msg_sent_counts;
    const ProtocolOptions&		m_options;
    std::vector<cbnsim::Predicate>	m_route_table;
    bool				m_route_table_dirty;
    unsigned int			m_srur_packets;
    std::map<unsigned int,sr_response>	m_sr_responses;
    
  public:
    CBCBNode( unsigned int nid, const BroadcastLayer& bl, const ProtocolOptions& options );

    virtual ssim::ProcessId activate();

    virtual void add_neighbor( ssim::ProcessId id, ssim::Time length );

    virtual size_t bytesize() const;

    void init();
    
    virtual void issue_packet( unsigned int ifid, const cbnsim::Packet* packet, ssim::Time lag = 0, bool count = true );

    virtual void issue_packet( ssim::ProcessId pid, const cbnsim::Packet* packet, ssim::Time lag = 0 )
    { cbnsim::Node::issue_packet( pid, packet, lag ); }

    void issue_predicate( bool force = false );

    void issue_sr();

    void process_packet( unsigned int ifid, const cbnsim::Packet& pkt );

    void reset_srur_packets()
    { m_srur_packets = 0; }

    virtual void send_message( cbnsim::MessageId mid, const Message& message );

    virtual void set_predicate( const cbnsim::Predicate& predicate, unsigned int, bool );

    unsigned int srur_packets() const
    { return m_srur_packets; }

    virtual bool has_failed() const { return false; };
    virtual bool count_power() const { return false; };
  };
};

#endif
