// -*- C++ -*-
//
//  This file is part of CBCBSIM, the Combined Broadcast Content-Based
//  (CBCB) routing simulation.
//
//  Authors: See the file AUTHORS for full details. 
//
//  Copyright (C) 2002-2004 University of Colorado
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,
//  USA, or send email to serl@cs.colorado.edu.
//
// $Id: CBCBSimulation.h,v 1.1 2004/03/19 17:04:48 carzanig Exp $
//
#ifndef CBCBSIM_CBCBSIMULATION_H
#define CBCBSIM_CBCBSIMULATION_H

#include <cbnsim/CBNSimulation.h>
#include "cbcbsim/MSTBroadcastLayer.h"
#include "cbcbsim/ProtocolOptions.h"
#include "cbcbsim/SPTsBroadcastLayer.h"

namespace cbcbsim
{
  class CBCBSimulation : public cbnsim::CBNSimulation
    {
      //MSTBroadcastLayer	m_bl;
      SPTsBroadcastLayer m_bl;
      ProtocolOptions	m_options;
      
    public:
      CBCBSimulation();

      const BroadcastLayer& broadcast_layer() const
	{ return m_bl; }

      void pip( ProtocolOptions::PredicateIssuePolicy pip )
      { m_options.pip = pip; }

      void pip_sec( ssim::Time pip_sec )
      { m_options.pip_sec = pip_sec; }

      void pre_run_simulation_hook() throw ();

      void prp( ProtocolOptions::PredicateRefreshPolicy prp )
      { m_options.prp = prp; }

      void prp_sec( ssim::Time prp_sec )
      { m_options.prp_sec = prp_sec; }

      void srsp( ProtocolOptions::SRSendPolicy srsp )
      { m_options.srsp = srsp; }

      void srsp_count( int srsp_count )
      { m_options.srsp_count = srsp_count; }

      void srsp_sec( ssim::Time srsp_sec )
      { m_options.srsp_sec = srsp_sec; }

      void setup_link_hook( ssim::ProcessId from, ssim::ProcessId to, unsigned int length ) throw ()
	{ m_bl.addLink( from, to, length ); }

      void setup_node_hook( cbnsim::Node* node ) throw ()
	{ m_bl.addNode( node->pid() ); }

      void setup_topology_hook() throw ();
    };
};

#endif
